import pytest


@pytest.mark.asyncio
class TestInterceptorComponent:
    async def test_interceptor(self):
        import aiohttp
        import lunar_interceptor  # type: ignore [reportUnusedImport]

        async with aiohttp.ClientSession() as sess:
            async with sess.get("https://httpbin.org/get") as resp:
                assert resp.status == 200

    async def test_interceptor_with_none_as_headers(self):
        import aiohttp
        import lunar_interceptor  # type: ignore [reportUnusedImport]

        async with aiohttp.ClientSession() as sess:
            async with sess.get("https://httpbin.org/get", headers=None) as resp:
                assert resp.status == 200



from lunar_interceptor.interceptor.interceptor import Interceptor
from lunar_interceptor.interceptor.configuration import (
    InterceptorConfig,
    ConnectionConfig,
    get_interceptor_config,
    ENV_LUNAR_PROXY_HOST_KEY,
)

__all__ = [
    "Interceptor",
    "InterceptorConfig",
    "ConnectionConfig",
    "get_interceptor_config",
    "ENV_LUNAR_PROXY_HOST_KEY",
]



from os import environ

import pytest

from lunar_interceptor.interceptor.helpers import load_env_value


@pytest.mark.asyncio
class TestHelpersComponent:
    async def test_load_env_value_no_env_found(self):
        value: int = load_env_value("test", int, 5)
        assert value == 5

    async def test_load_env_value_env_found(self):
        environ["LUNAR_TEST_VALUE_A"] = "1"
        value: int = load_env_value("LUNAR_TEST_VALUE_A", int, 5)
        assert value == 1



import logging
from time import sleep
from json import loads
from typing import Any, List, Dict, Optional

from lunar_interceptor.interceptor.hooks.const import *
from lunar_interceptor.interceptor.hooks.helpers import *
from lunar_interceptor.interceptor.fail_safe import FailSafe
from lunar_interceptor.interceptor.hooks.hook import LunarHook
from lunar_interceptor.interceptor.traffic_filter import TrafficFilter
from lunar_interceptor.interceptor.configuration import ConnectionConfig

_hook = True

try:
    import requests  # type: ignore [reportMissingModuleSource]

except ImportError:
    _hook = False


class RequestsHook(LunarHook):
    def __init__(
        self,
        logger: logging.Logger,
        fail_safe: FailSafe,
        traffic_filter: TrafficFilter,
        lunar_proxy_configuration: ConnectionConfig,
    ) -> None:
        self._connection_config = lunar_proxy_configuration
        self._logger = logger
        self._traffic_filter = traffic_filter
        self._fail_safe = fail_safe
        self._original_function = requests.Session.request  # type: ignore [reportPrivateUsage, reportUnknownMemberType]

        self._fail_safe.handle_on(
            (requests.ConnectionError,)  # type: ignore [reportOptionalCall]
        )

    @staticmethod
    def is_hook_supported() -> bool:
        return _hook

    def init_hooks(self) -> None:
        self._logger.debug("Initializing Requests Hook")
        requests.Session.request = self._hook_module()  # type: ignore [reportPrivateUsage, reportUnknownMemberType]

    def remove_hooks(self) -> None:
        self._logger.debug("Removing Requests Hook")
        requests.Session.request = self._original_function  # type: ignore [reportPrivateUsage, reportUnknownMemberType]

    async def make_connection(
        self, url: str, headers: Optional[Dict[str, str]]
    ) -> bool:
        try:
            response = requests.get(url, headers=headers)  # type: ignore [reportUnboundVariable]

            if response.status_code == 200:
                resp_json = loads(response.content)
                self._traffic_filter.managed = resp_json.get("managed", False)
                return True

        except Exception as e:
            self._logger.warning(f"Establishing handshake with Lunar Proxy. Error: {e}")

        return False

    def _hook_module(self):
        def _request(
            client_session: "requests.sessions.Session",
            method: str,
            url: str,
            *args: List[Any],
            **kwargs: Dict[str, Any],
        ) -> "requests.Response":
            lunar_req_id = generate_request_id()
            url_obj = URL(url)
            original_headers = kwargs.pop(HEADERS_KWARGS_KEY, {})

            with self._fail_safe:
                if self._fail_safe.state_ok and self._traffic_filter.is_allowed(
                    str(url_obj.host), original_headers
                ):
                    return self._make_request(
                        client_session=client_session,
                        method=method,
                        url_object=url_obj,
                        original_headers=original_headers,
                        lunar_req_id=lunar_req_id,
                        sequence_id=None,
                        *args,
                        **kwargs,
                    )

            kwargs[HEADERS_KWARGS_KEY] = original_headers
            self._logger.debug(
                f"Request {lunar_req_id} - Will send {url_obj} without using Lunar Proxy"
            )
            return self._original_function(
                self=client_session, method=method, url=url, *args, **kwargs
            )

        return _request

    def _make_request(
        self,
        client_session: "requests.sessions.Session",
        method: str,
        url_object: URL,
        original_headers: Dict[str, Any],
        lunar_req_id: str,
        sequence_id: Optional[str] = None,
        *args: List[Any],
        **kwargs: Dict[str, Any],
    ) -> "requests.Response":
        manipulated_headers = generate_modified_headers(
            original_url=url_object,
            original_headers=original_headers,
            sequence_id=sequence_id,
            lunar_tenant_id=self._connection_config.tenant_id,
            traffic_filter=self._traffic_filter,
            lunar_req_id=lunar_req_id,
        )
        modified_url = str(
            generate_modified_url(
                self._connection_config.proxy_scheme,
                self._connection_config.proxy_host_with_port,
                url_object,
            ),
        )
        self._logger.debug(
            f"Request {lunar_req_id} - Forwarding the request to {modified_url} using Lunar Proxy"
        )
        response = self._original_function(
            self=client_session,
            method=method,
            url=modified_url,
            headers=manipulated_headers,
            *args,
            **kwargs,
        )

        self._fail_safe.validate_headers(response.headers)

        retry_sequence_id = self._prepare_requests_for_retry(
            response.headers, lunar_req_id
        )
        if retry_sequence_id is not None:
            response = self._make_request(
                client_session=client_session,
                method=method,
                url_object=url_object,
                original_headers=original_headers,
                lunar_req_id=lunar_req_id,
                sequence_id=retry_sequence_id,
                *args,
                **kwargs,
            )

        modified_response_headers = self._generate_modified_response_headers(
            response.headers
        )
        response.headers = modified_response_headers  # type: ignore [reportPrivateUsage]

        return response

    def _prepare_requests_for_retry(
        self,
        headers: "requests.models.CaseInsensitiveDict[str]",
        lunar_req_id: str,
    ) -> Optional[str]:
        raw_retry_after = headers.get(LUNAR_RETRY_AFTER_HEADER_KEY)
        if raw_retry_after is None:
            return None

        sequence_id = headers.get(LUNAR_SEQ_ID_HEADER_KEY)
        if sequence_id is None:
            self._logger.debug(
                f"Request {lunar_req_id} - Retry required, but {LUNAR_SEQ_ID_HEADER_KEY} is missing!"
            )
            return None

        try:
            retry_after = float(raw_retry_after)
        except ValueError:
            self._logger.debug(
                f"Request {lunar_req_id} - Retry required, but parsing header {LUNAR_RETRY_AFTER_HEADER_KEY}"
                + f"as float failed ({raw_retry_after})"
            )
            return None

        self._logger.debug(
            f"Request {lunar_req_id} - Retry required, will retry in {retry_after} seconds..."
        )
        sleep(retry_after)
        return sequence_id

    def _generate_modified_response_headers(
        self, original_headers: "requests.models.CaseInsensitiveDict[str]"
    ) -> "requests.models.CaseInsensitiveDict[str]":
        if LUNAR_SEQ_ID_HEADER_KEY not in original_headers:
            return original_headers

        manipulated_headers = original_headers.copy()
        manipulated_headers.pop(LUNAR_SEQ_ID_HEADER_KEY)
        return requests.models.CaseInsensitiveDict(manipulated_headers)  # type: ignore [reportUnboundVariable]



import logging
import pkg_resources
from os import getenv
from typing import Callable, TypeVar

# Const
_T = TypeVar("_T")


def load_env_value(
    key: str,
    cast_to: Callable[[str], _T],
    default_value: _T,
    logger: logging.Logger = logging.getLogger(),
) -> _T:
    """Loads and prepare the values from the env variables, and cast in a type-safe manner.

    Args:
        key (str): The ENV value to get the value from.
        cast_to (_T): The type to cast the ENV value to.
        default_value (_T): Fallback value in case an error occurs.

    Returns:
        _T: The cast value or the default value passed to the function.
    """
    extracted_value = getenv(key, None)

    if extracted_value is None:
        logger.debug(
            f"Could not find value of: {key}.Will set {default_value} as default"
        )
        return default_value

    try:
        return cast_to(extracted_value)

    except ValueError as error:
        logger.warning(
            f"Could not convert the value of: {key} to: {type(cast_to)}, \
                Error: {error} Will set {default_value} as default"
        )

    return default_value


def get_package_version(package_name: str) -> str:
    try:
        return pkg_resources.get_distribution(package_name).version
    except Exception:
        return "unknown"



import asyncio
import logging
from typing import List

from lunar_interceptor.interceptor.hooks.const import *
from lunar_interceptor.interceptor.hooks import LUNAR_HOOKS
from lunar_interceptor.interceptor.fail_safe import FailSafe
from lunar_interceptor.interceptor.singleton import Singleton
from lunar_interceptor.interceptor.hooks.hook import LunarHook
from lunar_interceptor.interceptor.traffic_filter import TrafficFilter
from lunar_interceptor.interceptor.configuration import ConnectionConfig


class Interceptor(metaclass=Singleton):
    """
    This is the logic that allow the framework to forward the HTTP/S requests through Lunar Proxy.

    Args:
        lunar_proxy_host (str): The Lunar Proxy IP and port string, this allows the interceptor to correctly forward the requests through the relevant Proxy.
        lunar_handshake_port (str): The Lunar Proxy handshake port as string, this allows the interceptor to validate connection with the Proxy.
        proxy_support_tls (bool): If true, uses HTTPS communication with the proxy when the original request uses HTTPS. If false, all communication with proxy will be made using HTTP.
                                  Regardless of this argument, proxy will use the original scheme when connecting to external APIs.
        fail_safe (FailSafe): The FailSafe module.
        traffic_filter (TrafficFilter): The TrafficFilter module.
        logger (logging.Logger): Logger.
    """

    def __init__(
        self,
        lunar_proxy_configuration: ConnectionConfig,
        fail_safe: FailSafe,
        traffic_filter: TrafficFilter,
        logger: logging.Logger,
    ):
        self._connection_config = lunar_proxy_configuration
        self._traffic_filter = traffic_filter
        self._fail_safe = fail_safe
        self._logger = logger
        self._lunar_hooks: List[LunarHook] = []

    def set_hooks(self) -> None:
        """
        Initialize the required hooks to the AioHttp framework in order to allow traffic to be forwarded through Lunar Proxy.
        """
        for module in LUNAR_HOOKS:
            if module.is_hook_supported():
                self._lunar_hooks.append(
                    module(
                        lunar_proxy_configuration=self._connection_config,
                        logger=self._logger,
                        fail_safe=self._fail_safe,
                        traffic_filter=self._traffic_filter,
                    )
                )

        self._logger.debug("Python Interceptor is loaded...")

        asyncio.get_event_loop().run_until_complete(
            self._validate_lunar_proxy_connection()
        )

    async def _validate_lunar_proxy_connection(self) -> None:
        """
        Checks the communication to Lunar Proxy.
        """

        headers = {X_LUNAR_TENANT_ID_HEADER_KEY: self._connection_config.tenant_id}
        proxy_handshake_host = f"{self._connection_config.proxy_scheme}://{self._connection_config.proxy_host}:{self._connection_config.handshake_port}/handshake"

        if not self._lunar_hooks:
            return

        self._logger.debug("Establishing handshake with Lunar Proxy...")

        res = await self._lunar_hooks[0].make_connection(
            url=proxy_handshake_host, headers=headers
        )

        if not res:
            self._logger.warning(
                f"[ⓧ] Failed to communicate with Lunar Proxy.\n"
                f" Lunar Interceptor is disabled.\n"
                f" please make sure that Lunar Proxy is running and port '{self._connection_config.handshake_port}'"
                f" is set as the handshake port.\n"
                f" For more information please refer to:"
                f" http://docs.lunar.dev/installation-configuration/configuration#lunar-interceptor-configuration"
            )
            return

        for hook in self._lunar_hooks:
            hook.init_hooks()

        self._logger.info(
            "[ⓥ] Successfully communicate with Lunar Proxy"
            "Lunar Interceptor is ENABLED!"
        )



from typing import Any, Dict, Type


class Singleton(type):
    _instances: Dict[Type[Any], Any] = {}

    def __call__(cls, *args: Any, **kwargs: Any):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)

        return cls._instances[cls]



from logging import Logger
from typing import Optional, List
from dataclasses import dataclass, field

from lunar_interceptor.interceptor.hooks.const import VERSION, HTTPS_SCHEME, HTTP_SCHEME
from lunar_interceptor.interceptor.helpers import load_env_value

ENV_LUNAR_PROXY_HOST_KEY = "LUNAR_PROXY_HOST"
_LUNAR_TENANT_ID = "LUNAR_TENANT_ID"
_ENV_LUNAR_HANDSHAKE_PORT_KEY = "LUNAR_HANDSHAKE_PORT"
_ENV_PROXY_SUPPORT_TLS_KEY = "LUNAR_PROXY_SUPPORT_TLS"
_ENV_TRAFFIC_FILTER_ALLOW_LIST = "LUNAR_ALLOW_LIST"
_ENV_TRAFFIC_FILTER_BLOCK_LIST = "LUNAR_BLOCK_LIST"
_ENV_FAIL_SAFE_EXIT_COOLDOWN_SEC = "LUNAR_EXIT_COOLDOWN_AFTER_SEC"
_ENV_FAIL_SAFE_ENTER_AFTER = "LUNAR_ENTER_COOLDOWN_AFTER_ATTEMPTS"


@dataclass
class TrafficFilterConfig:
    allow_list: Optional[str] = load_env_value(
        _ENV_TRAFFIC_FILTER_ALLOW_LIST, str, None
    )
    block_list: Optional[str] = load_env_value(
        _ENV_TRAFFIC_FILTER_BLOCK_LIST, str, None
    )


@dataclass
class FailSafeConfig:
    cooldown_time: int = load_env_value(_ENV_FAIL_SAFE_ENTER_AFTER, int, 10)
    max_errors: int = load_env_value(_ENV_FAIL_SAFE_EXIT_COOLDOWN_SEC, int, 5)


@dataclass
class ConnectionConfig:
    is_valid: bool
    proxy_host: str
    proxy_port: int
    proxy_scheme: str
    proxy_url: str
    proxy_host_with_port: str
    tenant_id: str = load_env_value(_LUNAR_TENANT_ID, str, "unknown")
    handshake_port: str = load_env_value(_ENV_LUNAR_HANDSHAKE_PORT_KEY, str, "8081")


@dataclass
class InterceptorConfig:
    connection_config: ConnectionConfig
    version: str = VERSION
    traffic_filter: TrafficFilterConfig = field(default_factory=TrafficFilterConfig)
    fail_safe_config: FailSafeConfig = field(default_factory=FailSafeConfig)


def get_connection_config(logger: Logger) -> ConnectionConfig:
    proxy_host_value: str = load_env_value(ENV_LUNAR_PROXY_HOST_KEY, str, "null")
    proxy_scheme: str = (
        HTTPS_SCHEME
        if load_env_value(_ENV_PROXY_SUPPORT_TLS_KEY, int, 0)
        else HTTP_SCHEME
    )
    if proxy_host_value == "null":
        return ConnectionConfig(
            is_valid=False,
            proxy_host="",
            proxy_port=0,
            proxy_scheme=proxy_scheme,
            proxy_url="",
            proxy_host_with_port="",
        )

    proxy_host_and_port: List[str] = proxy_host_value.split(":")
    if len(proxy_host_and_port) == 0:
        logger.warning(
            "Could not obtain the Host value of Lunar Proxy from environment variables,"
            f"please set {ENV_LUNAR_PROXY_HOST_KEY} to the Lunar Proxy's host/IP and port in."
            f"current value: {proxy_host_value}"
        )
        return ConnectionConfig(
            is_valid=False,
            proxy_host="",
            proxy_port=0,
            proxy_scheme=proxy_scheme,
            proxy_url="",
            proxy_host_with_port="",
        )

    if len(proxy_host_and_port) == 1:
        logger.warning(
            "Could not obtain the Port value of Lunar Proxy from environment variables,"
            f"please set {ENV_LUNAR_PROXY_HOST_KEY} to the Lunar Proxy's host/IP and port in order to allow the interceptor to be loaded."
            f"current value: {proxy_host_value}"
        )
        return ConnectionConfig(
            is_valid=False,
            proxy_host="",
            proxy_port=0,
            proxy_scheme=proxy_scheme,
            proxy_url="",
            proxy_host_with_port="",
        )

    if len(proxy_host_and_port) > 2:
        logger.warning(
            "Could not parse the Host value of Lunar Proxy from environment variables,"
            f"please set {ENV_LUNAR_PROXY_HOST_KEY} to the Lunar Proxy's host/IP and port in with the format of 'host:port'."
            "Please note that the value should not contain any additional ':' such as Protocol in order to allow the interceptor to be loaded."
            f"current value: {proxy_host_value}"
        )
        return ConnectionConfig(
            is_valid=False,
            proxy_host="",
            proxy_port=0,
            proxy_scheme=proxy_scheme,
            proxy_url="",
            proxy_host_with_port="",
        )

    proxy_host: str = proxy_host_and_port[0]
    try:
        proxy_port: int = int(proxy_host_and_port[1])
    except ValueError:
        logger.warning(
            "Could not parse the Port value of Lunar Proxy from environment variables,"
            f"please set {ENV_LUNAR_PROXY_HOST_KEY} to the Lunar Proxy's host/IP and port in order to allow the interceptor to be loaded."
            f"current value: {proxy_host_value}"
        )
        return ConnectionConfig(
            is_valid=False,
            proxy_host="",
            proxy_port=0,
            proxy_scheme=proxy_scheme,
            proxy_url="",
            proxy_host_with_port="",
        )

    proxy_url = f"{proxy_scheme}://{proxy_host}:{proxy_port}"
    proxy_host_with_port = f"{proxy_host}:{proxy_port}"
    return ConnectionConfig(
        is_valid=True,
        proxy_host=proxy_host,
        proxy_port=proxy_port,
        proxy_scheme=proxy_scheme,
        proxy_url=proxy_url,
        proxy_host_with_port=proxy_host_with_port,
    )


def get_interceptor_config(logger: Logger) -> InterceptorConfig:
    proxy_connection_info: ConnectionConfig = get_connection_config(logger)
    return InterceptorConfig(proxy_connection_info)



import logging
from re import compile, match
from typing import Dict, Optional, List
from socket import gethostbyname, error as socket_error
from ipaddress import ip_address, IPv4Address, IPv4Network

# This regex check test: https://regex101.com/r/Zt9EyZ/1
_NOT_ONLY_NUMBERS = compile(r"^[\d.]+$")
_RE_ADDRESS_VALIDATOR = compile(
    r"^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9]){2,}$"
)

# Consts
_LIST_DELIMITER = ","
_ALLOW_LIST = "AllowList"
_BLOCK_LIST = "BlockList"
_ALLOWED_HEADER_VALUE = "true"
_ALLOWED_HEADER_KEY = "x-lunar-allow"
_PRIVATE_IP_RANGES: Dict[str, IPv4Network] = {
    "10": IPv4Network("10.0.0.0/8"),
    "12": IPv4Network("127.0.0.0/8"),
    "17": IPv4Network("172.16.0.0/12"),
    "19": IPv4Network("192.168.0.0/16"),
}
_BLACK_HOLE = IPv4Network("0.0.0.0/32")


class TrafficFilter:
    """
    TrafficFilter Mechanism assists to filter the traffic forwarded through the Proxy.

    Args:
        raw_block_list (Optional[str]): The block list allow the user to avoid forwarding
                                                selected hosts or IPs through Lunar Proxy
        raw_allow_list (Optional[str]): The allow list allow the user to only forward
                                                selected hosts or IPs through Lunar Proxy
        logger (logging.Logger): Logger.
    """

    def __init__(
        self,
        raw_block_list: Optional[str],
        raw_allow_list: Optional[str],
        logger: logging.Logger,
    ):
        self._logger = logger
        self._managed = False
        self._is_external_cache: Dict[str, bool] = dict()
        self._block_list: Optional[List[str]] = self._parse_list(raw_block_list)
        self._allow_list: Optional[List[str]] = self._parse_list(raw_allow_list)
        self._state_ok = self.is_access_list_valid()
        self._logger.debug(
            f"TrafficFilter loaded, TrafficFilter validation passed successfully:\
                 {self._state_ok}"
        )

    def is_allowed(self, host_or_ip: str, headers: Optional[Dict[str, str]]) -> bool:
        """Check if the given HOST or IP should be forward through the Proxy

        Args:
            host_or_ip (str): The HOST or IP to check for.

        Returns:
            bool: True if the given HOST or IP should be forward through the Proxy.
        """
        if not self._state_ok:
            return False

        header_based_filter = self._check_for_header_based_filter(headers)
        if header_based_filter is not None:
            return header_based_filter

        return self._check_if_host_or_ip_is_allowed(host_or_ip=host_or_ip)

    @property
    def managed(self) -> bool:
        """Check if the Proxy is managed

        Returns:
            bool: True if Proxy is managed else False.
        """
        return self._managed

    @managed.setter
    def managed(self, managed: bool = False) -> None:
        """Update the status of whether the proxy is managed and validate the list after the update.

        Returns:
            None
        """
        self._logger.debug(f"Proxy is running in managed={managed} mode")
        self._managed = managed

    def _check_if_host_or_ip_is_allowed(self, host_or_ip: str) -> bool:
        """Check if the given HOST or IP should be forward through the Proxy

        Args:
            host_or_ip (str): The HOST or IP to check for.

        Returns:
            bool: True if the given HOST or IP should be forward through the Proxy.
        """
        is_allowed = self._check_allowed(host_or_ip=host_or_ip)
        if is_allowed is not None:
            return is_allowed

        return self._check_blocked(host_or_ip) and self._is_external(host_or_ip)

    def _check_allowed(self, host_or_ip: str) -> Optional[bool]:
        """Check if the given HOST or IP should be forward through the Proxy

        Args:
            host_or_ip (str): The HOST or IP to check for.

        Returns:
            bool: True if the given HOST or IP should be forward through the Proxy.
        """
        if self._allow_list is None:
            return None

        return host_or_ip in self._allow_list

    def _check_blocked(self, host_or_ip: str) -> bool:
        """Check if the given HOST or IP should not be forward through the Proxy

        Args:
            host_or_ip (str): The HOST or IP to check for.

        Returns:
            bool: True if the given HOST or IP should not be forward through the Proxy.
        """
        if not self._block_list:
            return True

        if host_or_ip in self._block_list:
            return False

        return True

    def _parse_list(self, value_to_parse: Optional[str]) -> Optional[List[str]]:
        """Parsed the given value_to_parse if needed.

        Args:
            value_to_parse (Optional[str]): A string to parse to a List of strings.

        Returns:
            Optional[List[str]]: If exists, returns a list of strings or None if empty.
        """
        if value_to_parse is None or not value_to_parse:
            return None

        return value_to_parse.split(_LIST_DELIMITER)

    def _check_for_header_based_filter(
        self, headers: Optional[Dict[str, str]]
    ) -> Optional[bool]:
        if not headers:
            return None

        is_allowed = headers.pop(_ALLOWED_HEADER_KEY, None)

        if is_allowed is None:
            return is_allowed

        return is_allowed == _ALLOWED_HEADER_VALUE

    def is_access_list_valid(self) -> bool:
        """Validates the destinations lists.

        Returns:
            bool: True if the validation flow passed successfully, False otherwise.
        """
        if not self._validate_allow():
            return False

        if not self._validate_block():
            self._logger.warning(
                "Interceptor will be disable to avoid \
                 passing wrong traffic through the Proxy."
            )
            return False

        return True

    def _validate_allow(self) -> bool:
        """Validate the allow list values

        Returns:
            bool: True if the validation passed,
                  in the allow list we dont failed on wrong values
                  and continue the flow.
        """
        if not self._allow_list:
            return True

        values_to_remove: List[str] = []

        for host in self._allow_list:
            if not (self._validate_host(host) or self._validate_ip(host)):
                values_to_remove.append(host)

        for value_to_remove in values_to_remove:
            self._logger.warning(
                f"Unsupported value '{value_to_remove}' will removed from the allowed list."
            )
            self._allow_list.remove(value_to_remove)

        return True

    def _validate_block(self):
        """Validate the block list values

        Returns:
            bool: True if the validation passed, False otherwise.
        """
        if not self._block_list:
            return True

        if self._allow_list:
            self._logger.warning(
                f"TrafficFilter::Found {_ALLOW_LIST} skipping the {_BLOCK_LIST}"
            )
            self._block_list = []
            return True

        block_list_validation_pass = True
        for host in self._block_list:
            if not (self._validate_host(host) or self._validate_ip(host)):
                self._logger.warning(
                    f"Error while parsing '{host}' from the block list"
                )
                block_list_validation_pass = False

        return block_list_validation_pass

    def _validate_host(self, host: str) -> bool:
        """Validate that the given host is valid.

        Args:
            host (str): The host to validate

        Returns:
            bool: True if the host is valid, False otherwise.
        """
        if match(_NOT_ONLY_NUMBERS, host) is not None:
            return False

        return match(_RE_ADDRESS_VALIDATOR, host) is not None

    def _is_external(self, host_or_ip: str) -> bool:
        """Check whether an HOST or IP is external or not

        Args:
            host_or_ip (str): The HOST or IP to check for.

        Returns:
            bool: True if the HOST or IP is external, False otherwise
        """
        is_external = self._is_external_cache.get(host_or_ip, None)

        if is_external is not None:
            return is_external

        if self._validate_ip(host_or_ip):
            is_external = self._is_external_ip(host_or_ip)

        else:
            is_external = self._is_external_domain(host_or_ip)

        if is_external is None:
            # For cases that we could not resolve the destination.
            # We do not store the result to try again on next iteration.
            return False

        self._is_external_cache[host_or_ip] = is_external
        return is_external

    def _validate_ip(self, ip: str) -> bool:
        """Validate that the given IP is a valid address

        Args:
            ip (str): The ip to validate

        Returns:
            bool: True if the IP is valid, False otherwise.
        """
        try:
            ip_address(ip)
            return True

        except:
            return False

    def _is_external_ip(self, ip: str) -> bool:
        """Check whether an IP is external or not

        Args:
            ip (str): The IP to check for.

        Returns:
            bool: True if the IP is external, False otherwise
        """
        return IPv4Address(ip) not in _PRIVATE_IP_RANGES.get(ip[:2], _BLACK_HOLE)

    def _is_external_domain(self, host: str) -> Optional[bool]:
        """Check whether an HOST is external or not

        Args:
            host (str): The HOST to check for.

        Returns:
            bool: True if the HOST is external, False otherwise
        """
        try:
            return self._is_external_ip(gethostbyname(host))

        except socket_error as error:
            # If there is a network error, we will avoid storing this and will try again next time.
            self._logger.warning(
                f"TrafficFilter::Could not resolve: '{host}'. Error: {error}"
            )
            return None



import logging

import pytest

from lunar_interceptor.interceptor.traffic_filter import TrafficFilter

_LOGGER = logging.getLogger()


@pytest.mark.asyncio
class TestInterceptorTrafficFilterComponent:
    async def test_traffic_filter_parser_with_valid_values(self):
        raw_allow_list = "www.google.com,google.com,192.168.24.24"
        raw_block_list = "www.no.com,www.no-no.net"

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert traffic_filter.is_allowed("www.google.com", None)
        assert traffic_filter.is_allowed("google.com", None)
        assert traffic_filter.is_allowed("192.168.24.24", None)

        assert not traffic_filter.is_allowed("www.no.com", None)
        assert not traffic_filter.is_allowed("www.no-no.net", None)

        assert not traffic_filter.is_allowed("www.should-block.net", None)
        assert traffic_filter.is_allowed(
            "www.not-allowed.net", {"x-lunar-allow": "true"}
        )

    async def test_traffic_filter_parser_with_invalid_url_values(self):
        raw_allow_list = "www.google.c,google.com"
        raw_block_list = "www.no.com,wno."

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False
        assert not traffic_filter.is_allowed("www.google.com", None)
        assert traffic_filter.is_allowed("google.com", None)

        assert not traffic_filter.is_allowed("www.no.com", None)
        assert not traffic_filter.is_allowed("www.no-no.net", None)

        assert not traffic_filter.is_allowed("www.should-block.net", None)
        assert not traffic_filter.is_allowed("www.not-allowed.net", None)

        assert not traffic_filter.is_allowed("www.google.c", None)
        assert not traffic_filter.is_allowed("wno.", None)

    async def test_traffic_filter_parser_with_1_invalid_and_1_valid_ip_value_on_allow_list(
        self,
    ):
        raw_allow_list = "192.125,124.4555,127.0.0.1"
        raw_block_list = ""

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False
        assert not traffic_filter.is_allowed("www.google.com", None)
        assert not traffic_filter.is_allowed("google.com", None)

        assert not traffic_filter.is_allowed("192.125", None)
        assert not traffic_filter.is_allowed("124.4555", None)
        assert not traffic_filter.is_allowed("192.125,124.4555", None)

        assert not traffic_filter.is_allowed("www.should-block.net", None)
        assert not traffic_filter.is_allowed("www.not-allowed.net", None)

        assert not traffic_filter.is_allowed("www.google.c", None)
        assert not traffic_filter.is_allowed("wno.", None)
        assert traffic_filter.is_allowed("127.0.0.1", None)

    async def test_traffic_filter_parser_with_1_invalid_value_on_allow_list(self):
        raw_allow_list = "192.125,124.4555"
        raw_block_list = ""

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert not traffic_filter.is_allowed("www.google.com", None)
        assert not traffic_filter.is_allowed("google.com", None)

        assert not traffic_filter.is_allowed("192.125", None)
        assert not traffic_filter.is_allowed("124.4555", None)
        assert not traffic_filter.is_allowed("192.125,124.4555", None)

        assert not traffic_filter.is_allowed("www.should-block.net", None)
        assert not traffic_filter.is_allowed("www.not-allowed.net", None)

        assert not traffic_filter.is_allowed("www.google.c", None)
        assert not traffic_filter.is_allowed("wno.", None)
        assert not traffic_filter.is_allowed("127.0.0.1", None)

    async def test_traffic_filter_parser_with_invalid_ip_value_on_block_list(self):
        raw_allow_list = ""
        raw_block_list = "192.21,192.168.24.1"

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert not traffic_filter.is_allowed("www.google.com", None)
        assert not traffic_filter.is_allowed("google.com", None)

        assert not traffic_filter.is_allowed("192.125", None)
        assert not traffic_filter.is_allowed("124.4555", None)

        assert not traffic_filter.is_allowed("www.should-block.net", None)
        assert not traffic_filter.is_allowed("www.not-allowed.net", None)

        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("wno.", None)

    async def test_traffic_filter_with_block_list(self):
        raw_allow_list = None
        raw_block_list = (
            "192.168.24.1,google.com,https://www.facebook.com,https://httbin.com"
        )

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("google.com", None)

        assert not traffic_filter.is_allowed("https://www.facebook.com", None)
        assert not traffic_filter.is_allowed("https://httbin.com", None)

    async def test_traffic_filter_with_allow_list(self):
        raw_allow_list = "192.168.24.1,google.com,www.facebook.com,httpbin.com, httpbin"
        raw_block_list = None

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert traffic_filter.is_allowed("192.168.24.1", None)
        assert traffic_filter.is_allowed("google.com", None)

        assert traffic_filter.is_allowed("www.facebook.com", None)
        assert traffic_filter.is_allowed("httpbin.com", None)
        assert not traffic_filter.is_allowed("httpbin", None)

    async def test_traffic_filter_with_same_host_on_both_lists(self):
        raw_allow_list = "google.com"
        raw_block_list = "google.com"

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert traffic_filter.is_allowed("google.com", None)

    async def test_traffic_filter_should_not_work_when_using_protocol(self):
        raw_allow_list = None
        raw_block_list = "https://google.com"

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("google.com", None)

    async def test_traffic_filter_should_not_work_when_using_path_param(self):
        raw_allow_list = None
        raw_block_list = "google.com/my_test/1"

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("google.com", None)

    async def test_traffic_filter_should_not_work_when_using_port(self):
        raw_allow_list = None
        raw_block_list = "google.com:443"

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )

        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("google.com", None)

    async def test_traffic_filter_should_not_allow_internal_address(
        self,
    ):
        raw_allow_list = None
        raw_block_list = None

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("127.0.0.1", None)
        assert not traffic_filter.is_allowed("10.0.0.138", None)
        assert not traffic_filter.is_allowed("172.16.25.1", None)
        assert traffic_filter.is_allowed("google.com", None)
        assert traffic_filter.is_allowed("www.httpbin.com", None)

    async def test_traffic_filter_validate_cach_dont_fail(
        self,
    ):
        raw_allow_list = None
        raw_block_list = None

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("127.0.0.1", None)
        assert not traffic_filter.is_allowed("10.0.0.138", None)
        assert not traffic_filter.is_allowed("172.16.25.1", None)
        assert traffic_filter.is_allowed("google.com", None)
        assert traffic_filter.is_allowed("www.httpbin.com", None)
        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("127.0.0.1", None)
        assert not traffic_filter.is_allowed("10.0.0.138", None)
        assert not traffic_filter.is_allowed("172.16.25.1", None)
        assert traffic_filter.is_allowed("google.com", None)
        assert traffic_filter.is_allowed("www.httpbin.com", None)
        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("127.0.0.1", None)
        assert not traffic_filter.is_allowed("10.0.0.138", None)
        assert not traffic_filter.is_allowed("172.16.25.1", None)
        assert traffic_filter.is_allowed("google.com", None)
        assert traffic_filter.is_allowed("www.httpbin.com", None)

    async def test_traffic_filter_should_allow_internal_address_if_set_in_allow_list(
        self,
    ):
        raw_allow_list = "192.168.24.1"
        raw_block_list = None

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("192.168.24.2", None)

    async def test_traffic_filter_should_block_unresolved_adresses(
        self,
    ):
        raw_allow_list = None
        raw_block_list = None

        traffic_filter = TrafficFilter(
            raw_allow_list=raw_allow_list, raw_block_list=raw_block_list, logger=_LOGGER
        )
        traffic_filter.managed = False

        assert not traffic_filter.is_allowed("192.168.24.1", None)
        assert not traffic_filter.is_allowed("192.168.24.2", None)
        assert not traffic_filter.is_allowed("www.not_working.lol", None)



from typing import Dict, Optional

from yarl import URL

from .const import *
from lunar_interceptor.interceptor.traffic_filter import TrafficFilter
import uuid


def generate_request_id() -> str:
    """
    Generate a Request Identifier.

    Returns:
        str: A string representation of the generated ID.
    """
    return str(uuid.uuid4())


def generate_modified_headers(
    original_url: URL,
    original_headers: Optional[Dict[str, str]],
    sequence_id: Optional[str],
    traffic_filter: TrafficFilter,
    lunar_tenant_id: str,
    lunar_req_id: str,
) -> Dict[str, str]:
    """
    Generate the headers to support the proxy work flow.

    Args:
        original_url (yarl.URL):The url object converted by the original url passed to the request.
        original_headers (Dict[str ,str]): The dictionary of the original request headers.

    Returns:
        Dict[str, str]: A modified dictionary of headers.
    """
    if original_url.host is None:
        raise Exception("Could not parse the hostname, reverting to original flow.")

    destination_port = original_url.port

    host = (
        f"{original_url.host}:{destination_port}"
        if destination_port is not None
        else original_url.host
    )

    modified_headers: Dict[str, str] = original_headers if original_headers else {}

    modified_headers[X_LUNAR_HOST_HEADER_KEY] = host
    modified_headers[X_LUNAR_REQ_ID_HEADER_KEY] = lunar_req_id
    modified_headers[X_LUNAR_SCHEME_HEADER_KEY] = original_url.scheme
    modified_headers[X_LUNAR_INTERCEPTOR_HEADER_KEY] = LUNAR_INTERCEPTOR_HEADER_VALUE
    if traffic_filter.managed:
        modified_headers[X_LUNAR_TENANT_ID_HEADER_KEY] = lunar_tenant_id

    if sequence_id is not None:
        modified_headers[LUNAR_SEQ_ID_HEADER_KEY] = sequence_id

    return modified_headers


def generate_modified_url(scheme: str, proxy_host: str, original_url: URL) -> URL:
    """
    Generate the required url to connect through the proxy instead of the original host.

    Args:
        original_url (yarl.URL): The url object converted by the original url passed to the request.

    Returns:
        URL: An url object that point to Lunar proxy.
    """
    return original_url.with_scheme(scheme).with_host(proxy_host)



import logging
from datetime import datetime

import pytest
from freezegun import freeze_time

from lunar_interceptor.interceptor.fail_safe import FailSafe


class ExceptionTestA(Exception):
    pass


class ExceptionTestB(Exception):
    pass


_DELTA_TIME = 10
_COOLDOWN_TIME = 2
_MAX_ERROR_ALLOWED = 3


@pytest.mark.asyncio
class TestInterceptorFailSafeComponent:
    async def test_enter_fail_safe(self):
        fail_safe = FailSafe(
            _COOLDOWN_TIME, _MAX_ERROR_ALLOWED, logging.getLogger(), (ExceptionTestA,)
        )

        assert fail_safe.state_ok

        for attempt in range(_MAX_ERROR_ALLOWED):
            with fail_safe:
                raise ExceptionTestA(f"Attempt - {attempt}")

        assert fail_safe.state_ok == False

    async def test_exit_fail_safe_after_cooldown(self):
        initial_datetime = datetime.now()
        with freeze_time(initial_datetime) as frozen_time:
            fail_safe = FailSafe(
                _COOLDOWN_TIME,
                _MAX_ERROR_ALLOWED,
                logging.getLogger(),
                (ExceptionTestB,),
            )

            for attempt in range(_MAX_ERROR_ALLOWED):
                with fail_safe:
                    raise ExceptionTestB(f"Attempt - {attempt}")

            assert fail_safe.state_ok == False

            frozen_time.tick(_DELTA_TIME)  # type: ignore [reportUnknownVariableType]

            assert fail_safe.state_ok

    async def test_fail_safe_handle_requierd_exceptions(self):
        fail_safe = FailSafe(
            _COOLDOWN_TIME, _MAX_ERROR_ALLOWED, logging.getLogger(), (ExceptionTestA,)
        )

        with fail_safe:
            raise ExceptionTestA("Attempt")

        with pytest.raises(ExceptionTestB):
            with fail_safe:
                raise ExceptionTestB("Attempt")

    async def test_fail_safe_enter_again_to_cooldown_after_first_connection_error(self):
        initial_datetime = datetime.now()
        with freeze_time(initial_datetime) as frozen_time:
            fail_safe = FailSafe(
                _COOLDOWN_TIME,
                _MAX_ERROR_ALLOWED,
                logging.getLogger(),
                (ExceptionTestA,),
            )

            assert fail_safe.state_ok

            for attempt in range(_MAX_ERROR_ALLOWED):
                with fail_safe:
                    raise ExceptionTestA(f"Attempt - {attempt}")

            assert fail_safe.state_ok == False

            frozen_time.tick(_DELTA_TIME)  # type: ignore [reportUnknownVariableType]

            assert fail_safe.state_ok

            with fail_safe:
                raise ExceptionTestA(f"Test")

            assert fail_safe.state_ok == False

    async def test_fail_safe_does_not_enter_again_to_cooldown_after_first_connection_succeeded(
        self,
    ):
        initial_datetime = datetime.now()
        with freeze_time(initial_datetime) as frozen_time:
            fail_safe = FailSafe(
                _COOLDOWN_TIME,
                _MAX_ERROR_ALLOWED,
                logging.getLogger(),
                (ExceptionTestA,),
            )

            assert fail_safe.state_ok

            for attempt in range(_MAX_ERROR_ALLOWED):
                with fail_safe:
                    raise ExceptionTestA(f"Attempt - {attempt}")

            assert fail_safe.state_ok == False

            frozen_time.tick(_DELTA_TIME)  # type: ignore [reportUnknownVariableType]

            assert fail_safe.state_ok

            with fail_safe:
                pass

            assert fail_safe.state_ok



from typing import Dict, Optional
from abc import ABC, abstractmethod


class LunarHook(ABC):
    @staticmethod
    @abstractmethod
    def is_hook_supported() -> bool:
        pass

    @abstractmethod
    def init_hooks(self) -> None:
        pass

    @abstractmethod
    def remove_hooks(self) -> None:
        pass

    @abstractmethod
    async def make_connection(
        self, url: str, headers: Optional[Dict[str, str]]
    ) -> bool:
        pass



import logging
from time import time
import traceback as tb
from types import TracebackType
from typing import Optional, Tuple, Type, Any, Mapping

from lunar_interceptor.interceptor.hooks.const import LUNAR_PROXY_ERROR_TRANSLATOR

# Default values
_DEFAULT_MAX_ERROR_ALLOWED = 5
_DEFAULT_FAILSAFE_COOLDOWN_SEC = 10

_HEADER_ERROR_KEY = "x-lunar-error"


class ProxyErrorException(Exception):
    pass


class FailSafe:
    """
    FailSafe Mechanism assists with creating a dynamic circuit breaker,
    preventing unnecessary delays and ensuring that the system runs smoothly.

    Args:
        cooldown_time (Optional[int]): the time of cooldown on max error exceeded.
        max_errors_allowed (Optional[int]): the amount of error to allow
                                            before enter to cooldown.
        logger (logging.Logger): Logger.
        count_on_exceptions (Tuple[Type[BaseException], ...]):
            The types of exception to catch, and not return to the user.
    """

    def __init__(
        self,
        cooldown_time: Optional[int],
        max_errors_allowed: Optional[int],
        logger: logging.Logger,
        handle_on: Tuple[Type[BaseException], ...] = (),
    ) -> None:
        self._logger = logger
        self._state_ok = True

        self._error_counter = 0
        self._cooldown_started_at = 0
        self._handle_on: Tuple[Type[BaseException], ...] = handle_on

        self._max_errors_allowed: int = cooldown_time or _DEFAULT_MAX_ERROR_ALLOWED
        self._cooldown_time: int = max_errors_allowed or _DEFAULT_FAILSAFE_COOLDOWN_SEC

    def __enter__(self) -> "FailSafe":
        return self

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Any,
        traceback: TracebackType,
    ) -> bool:
        if exc_type is not None and issubclass(exc_type, self._handle_on):
            self._on_error()
            self._logger.warning(
                f"FailSafe::Error communicating with Lunar Proxy, Error: {exc_value}"
            )
            if exc_type is not ProxyErrorException:
                self._logger.debug(
                    f"Exception: {str(exc_type)}, Traceback: {tb.format_tb(traceback)}"
                )

            return True  # Handle the exception for the caller

        elif exc_type is not None:
            return False  # Dont handle the exception for the caller

        self._error_counter = 0
        return True

    @property
    def state_ok(self) -> bool:
        """Get indication about the current FailSafe state.

        Returns:
            bool: True if the flow runs as expected,
                  False if circuit should be broken.
        """
        self._ensure_exit_fail_safe()
        return self._state_ok

    def handle_on(self, handle_on: Tuple[Type[BaseException], ...]):
        """Sets the type of exception the failsafe should handle

        Args:
            handle_on (Tuple[Type[BaseException], ...]): The exception types to handle
        """
        self._handle_on = self._handle_on + handle_on

    def validate_headers(self, headers: Mapping[str, str]):
        """Validates the response headers and if the error header exists then raises an Exception.

        Args:
            headers (Mapping[str, str]): The response headers to check if the error header exists

        Raises:
            ProxyErrorException: if the error header exists.
        """
        if _HEADER_ERROR_KEY not in headers:
            return

        errorCode = headers[_HEADER_ERROR_KEY]
        errorMsg = LUNAR_PROXY_ERROR_TRANSLATOR.get(
            errorCode,
            "Unknown error, Please check the Proxy logs for more information.",
        )

        self._logger.warning(
            "FailSafe::Error communicating with Lunar Proxy,"
            f"Error: {errorCode}"
            f"Message: {errorMsg}"
        )

        raise ProxyErrorException("An error occurs on the Proxy side")

    def _on_error(self):
        """
        Increase the counter and break the circuit if needed.
        """
        self._error_counter += 1
        self._ensure_enter_fail_safe()

    def _ensure_enter_fail_safe(self):
        """
        Ensure that the circuit breaks if needed.
        """
        if self._max_errors_allowed > self._error_counter:
            return

        self._state_ok = False
        self._cooldown_started_at = time()

    def _ensure_exit_fail_safe(self):
        """
        Ensure that the circuit is restored to the original flow if needed.
        """
        if (
            not self._state_ok
            and (time() - self._cooldown_started_at) >= self._cooldown_time
        ):
            self._state_ok = True



import logging
import platform

from lunar_interceptor.interceptor import (
    Interceptor,
    InterceptorConfig,
    get_interceptor_config,
    ENV_LUNAR_PROXY_HOST_KEY,
)
from lunar_interceptor.interceptor.helpers import load_env_value
from lunar_interceptor.interceptor.traffic_filter import TrafficFilter
from lunar_interceptor.interceptor.fail_safe import (
    FailSafe,
    ProxyErrorException,
)

_INTERCEPTOR_NAME = "lunar-interceptor"
_ENV_LUNAR_INTERCEPTOR_LOG_LEVEL = "LUNAR_INTERCEPTOR_LOG_LEVEL"


def _initialize_lunar_logger() -> logging.Logger:
    log_format = logging.Formatter(
        f"%(asctime)s - {_INTERCEPTOR_NAME} - %(levelname)s: %(message)s"
    )
    log_level = load_env_value(_ENV_LUNAR_INTERCEPTOR_LOG_LEVEL, str, "INFO")

    logger = logging.getLogger(name=_INTERCEPTOR_NAME)
    logger.setLevel(log_level)

    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(log_level)
    stream_handler.setFormatter(log_format)
    logger.addHandler(stream_handler)

    return logger


_LOGGER = _initialize_lunar_logger()
interceptor_config: InterceptorConfig = get_interceptor_config(_LOGGER)


def _load_fail_safe() -> FailSafe:
    fail_safe = FailSafe(
        cooldown_time=interceptor_config.fail_safe_config.cooldown_time,
        max_errors_allowed=interceptor_config.fail_safe_config.max_errors,
        logger=_LOGGER,
        handle_on=(ProxyErrorException,),
    )

    return fail_safe


def _build_traffic_filter_from_env_vars() -> TrafficFilter:
    return TrafficFilter(
        interceptor_config.traffic_filter.block_list,
        interceptor_config.traffic_filter.allow_list,
        _LOGGER,
    )


def _initialize_hooks():
    lunar_interceptor = Interceptor(
        lunar_proxy_configuration=interceptor_config.connection_config,
        fail_safe=_load_fail_safe(),
        traffic_filter=_build_traffic_filter_from_env_vars(),
        logger=_LOGGER,
    )
    lunar_interceptor.set_hooks()


if _LOGGER.isEnabledFor(logging.DEBUG):
    _LOGGER.debug(
        "Lunar Interceptor has loaded in debug mode."
        "The current configuration are"
        f"  * Interceptor Version: {interceptor_config.version}"
        f"  * Lunar Proxy Host: {interceptor_config.connection_config.proxy_host}"
        f"  * Lunar Proxy Handshake Port: {interceptor_config.connection_config.handshake_port}"
        ""
        "Environment details:"
        f"  * Python Engine Version: {platform.python_version()}"
    )

if (
    interceptor_config.connection_config.proxy_host != ""
):  # Lunar Interceptor can be loaded.
    _initialize_hooks()

else:  # Lunar Interceptor can׳t be loaded without the Lunar Proxy address.
    _LOGGER.warning(
        "Could not obtain the Host value of Lunar Proxy from environment variables,"
        f"please set {ENV_LUNAR_PROXY_HOST_KEY} to the Lunar Proxy's host/IP and port in"
        "order to allow the interceptor to be loaded."
        "Lunar Interceptor is DISABLED!"
    )



from lunar_interceptor.interceptor.helpers import get_package_version

HTTP_SCHEME = "http"
HTTPS_SCHEME = "https"
X_LUNAR_HOST_HEADER_KEY = "x-lunar-host"
X_LUNAR_SCHEME_HEADER_KEY = "x-lunar-scheme"
X_LUNAR_INTERCEPTOR_HEADER_KEY = "x-lunar-interceptor"
X_LUNAR_TENANT_ID_HEADER_KEY = "x-lunar-tenant-id"
X_LUNAR_REQ_ID_HEADER_KEY = "x-lunar-req-id"
INTERCEPTOR_TYPE_VALUE = "lunar-py-interceptor"
VERSION = get_package_version("lunar-interceptor")
INTERCEPTOR_HEADER_DELIMITER = "/"
LUNAR_INTERCEPTOR_HEADER_VALUE = (
    f"{INTERCEPTOR_TYPE_VALUE}{INTERCEPTOR_HEADER_DELIMITER}{VERSION}"
)

HEADERS_KWARGS_KEY = "headers"
CACHE_HEADERS_KEY = "headers"
LUNAR_SEQ_ID_HEADER_KEY = "x-lunar-sequence-id"
LUNAR_RETRY_AFTER_HEADER_KEY = "x-lunar-retry-after"

LUNAR_PROXY_ERROR_TRANSLATOR = {
    "1": "Wrong request, Lunar Proxy could not find header `x-lunar-host` and Proxy was not set to use query params.",
    "2": "The endpoint cannot be reached",
    "3": "Gateway timeout",
    "4": "Lunar Proxy could not find the endpoint",
    "5": "Lunar Proxy could not resolve host",
}
